import os 

SHARED_PATH = os.path.dirname(os.path.abspath(__file__)) + "/share"
ENV_FILE_PATH = SHARED_PATH + "/worker.env"
IMAGE = "ainblockchain/ain-worker-staging:latest"
VERSION="1.4.33"
TRACKER_ADDR = "http://52.79.226.211:3031/tracker"


import subprocess
import dotenv 
import requests_unixsocket
import os
from ain.instanceUtil import InstanceUtil 
import ain.constants as constants

class Worker():

  def __init__(self):
    self.ENVS = dotenv.dotenv_values(constants.ENV_FILE_PATH)
    self.session = requests_unixsocket.Session()
    self.hostTtydSocket = constants.SHARED_PATH + '/ain_worker_ttyd.sock'
    sharedPath = constants.SHARED_PATH.replace("/", '%2F')
    self.workerAddress =  'http+unix://' + sharedPath + '%2Fain_worker.sock'
    
  def start(self, optionRun):
    
    for key in ["NAME", "LIMIT_COUNT", "PRICE", "MNEMONIC", "SERVER_IP", "GPU"]:
      if self.ENVS[key].replace(" ", "") == "" and optionRun[key] == "":
        print('[-] ' + key + '- empty')
        return

      if optionRun[key] != "":
        dotenv.set_key(constants.ENV_FILE_PATH, key, optionRun[key])
        
    if optionRun["DESCRIPTION"] != "" and optionRun["DESCRIPTION"] != "empty":
      dotenv.set_key(constants.ENV_FILE_PATH, "DESCRIPTION", optionRun["DESCRIPTION"])
    print("[?] Do you want to start? (y/n)")
    answer = input()
    if (answer.lower() != 'y'):
      return
    
    # open provider's ttys socket 
    InstanceUtil.createTtyd(self.hostTtydSocket)

    # open docker container for ain worker server
    InstanceUtil.createContainer(constants.IMAGE, optionRun["GPU"])


  def status(self):
    try:
      response = self.session.get(self.workerAddress + "/info")
      ids = response.json()['id']
      if(len(ids) == 0):
        print("[+] does not exist")
        return
    except Exception as e:
      print("[-] worker server error")
      print(e)
      exit(1)

    print("[+] Status: Running")
    try:
      option = " ".join(ids)
      subprocess.run(["docker", "stats" , option])

    except Exception as e:
      print("[-] subprocess(docker) error")
      print(e)

  def stop(self):
    try:      
      response = self.session.get(self.workerAddress + "/info")
      cnt = response.json()['cnt']
      if (cnt != 0):
        print("[+] Instance count:" +str(cnt))
        
      print("[?] Do you want to stop? (y/n)")
      answer = input()
      if (answer.lower() != 'y'):
        return
    except Exception as e:
      print("[-] worker server error - info")

    try:
      response = self.session.get(self.workerAddress + "/terminate")
    except Exception as e:
      print("[-] worker server error - terminate")

    try:
      InstanceUtil.removeContainer("ain_worker")
      print('[+] succeded to remove container!')

    except Exception as e:
      print("[-] subprocess(docker) error")
      print(e)

    try:
      InstanceUtil.removeTtyd(self.hostTtydSocket)
      InstanceUtil.removeTtyd(constants.SHARED_PATH + "/ain_worker.sock")
      print('[+] succeded to remove ttyd socket')
    except Exception as e:
      print("[-] subprocess error(ttyd socker remove)")
      print(e)
      exit(1)

  def log(self):
    basePath = constants.SHARED_PATH + "/log/" 
    if (os.path.exists(basePath + "log.env")):
      os.remove(basePath + "log.env")
      
    logFileName = sorted(os.listdir(basePath))
    targetFile = logFileName[-1]
    if (len(logFileName) == 0):
      print("[+] does not exist")
      return
  
    print("[?] do you want to see recent log?(y/n)")
    answer = input()
    if (answer == "n"):
      for i in range(len(logFileName)):
        print(str(i) + " - " + logFileName[i])
      print("[?] input number")
      number = int(input())
      targetFile = logFileName[number]
    
    try:
      path = os.path.join(basePath, targetFile)
      if (answer):
        subprocess.run(["tail", "-f" , path])
      else:
        subprocess.run(["cat", path])
    except Exception as e:
      print("[-] subprocess(log) error")
      print(e)
      
    return

  def init(self):
    for key in ["NAME", "LIMIT_COUNT", "PRICE", "DESCRIPTION", "MNEMONIC", "SERVER_IP", "GPU"]:
      dotenv.set_key(constants.ENV_FILE_PATH, key, "")


import click
from ain.workerMgr import Worker
import ain.constants as constants
import requests

def checkVersion():
  MESSAGE = {
    "jsonrcp": "2.0",
    "method": "ain_checkCliVersion",
    "params": {
      "version": constants.VERSION
    },
    "id": 1
  }

  try:
    response = requests.get(constants.TRACKER_ADDR, data=MESSAGE)
    if (response.json()['result'] == 0): return True
    else: return False
  except Exception as e:
    print("[-] tracker server error")
    print(e)
    return False


@click.group()
@click.option('--type/--no-type', default=False)

def call(type):
  pass

@call.command()
@click.argument("command", type=click.Choice(['start', 'stop', 'status', 'log', 'init', 'version']))
@click.option('--name', default="", help="Ain Worker name")
@click.option('--max-instance-count ', default="", help="maximum number of Instances")
@click.option('--price', default="", help="ain/h")
@click.option('--mnemonic', default="", help="mnemonic")
@click.option('--description', default="empty", help="description")
@click.option('--server-ip', default="empty", help="Cloud Server IP")
@click.option('--gpu', default="false", help="gpu")

def worker(command, name, max_instance_count, price, mnemonic, description, server_ip, gpu):

  optionRun = {
    'NAME': name,
    'LIMIT_COUNT': max_instance_count,
    'MNEMONIC': mnemonic,
    'PRICE': price,
    'DESCRIPTION': description,
    'GPU': gpu,
    'SERVER_IP': server_ip
  }

  if checkVersion() == False:
    print('[!] you have to input "sudo pip3 install --upgrade ain-worker"')
    exit(1)
        
  w = Worker()  
  if (command == "start"):
    w.start(optionRun)
  elif (command == "stop"):
    w.stop()
  elif (command == "status"):
    w.status()
  elif (command == "log"):
    w.log()
  elif (command == "init"):
    w.init()
  elif (command == "version"):
    print(constants.VERSION)
    
if __name__ == '__main__':
  call()
